package com.detectdoc.opencv_document_detection.crop

import android.content.ContentValues.TAG
import android.content.Context
import android.graphics.Bitmap
import android.graphics.Matrix
import android.os.AsyncTask
import android.util.Log
import android.view.View
import android.widget.ImageView
import com.detectdoc.opencv_document_detection.Constants
import com.detectdoc.opencv_document_detection.SourceManager
import com.detectdoc.opencv_document_detection.processor.Corners
import com.detectdoc.opencv_document_detection.processor.cropPicture
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import org.opencv.android.Utils
import org.opencv.core.Mat

const val IMAGES_DIR = "smart_scanner"

class CropPresenter(val context: Context, private val iCropView: ICropView.Proxy) {
    private val picture: Mat? = SourceManager.pic
    val CROP_CODE = 1520
    private val corners: Corners? = SourceManager.corners
    private var croppedPicture: Mat? = null
    private var croppedBitmap: Bitmap? = null
    var angle = 90
    var isCropped = false
//    var moreRotateNeeded = false
    var isWidthMax = false
//    var lastCorners :Corners? = null

    init {
        iCropView.getPaperRect().onCorners2Crop(corners, picture?.size())
//        lastCorners = corners
        val bitmap = Bitmap.createBitmap(picture?.width() ?: Constants.width, picture?.height()
                ?: Constants.height, Bitmap.Config.ARGB_8888)
        if(bitmap.height>bitmap.width){
            isWidthMax =false
        }
        else{
            isWidthMax = true
        }

        Utils.matToBitmap(picture, bitmap, true)
        iCropView.getPaper().setImageBitmap(bitmap)
    }

    fun initAgain(){
        iCropView.getPaperRect().onCorners2Crop(corners, picture?.size())
    }

    fun crop() {
        if (picture == null) {
            Log.i(TAG, "picture null?")
            return
        }

//        if (croppedBitmap != null) {
//            Log.i(TAG, "already cropped")
//            return
//        }

        Observable.create<Mat> {
            it.onNext(cropPicture(picture, iCropView.getPaperRect().getCorners2Crop()))
        }
                .subscribeOn(Schedulers.computation())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe { pc ->
                    Log.i(TAG, "cropped picture: " + pc.toString())
                    croppedPicture = pc
                    croppedBitmap = Bitmap.createBitmap(pc.width(), pc.height(), Bitmap.Config.ARGB_8888)
                    Utils.matToBitmap(pc, croppedBitmap)

                    rotateBitmap(180.toFloat())
//                    iCropView.getCroppedPaper().scaleType = ImageView.ScaleType.FIT_XY
//                    iCropView.getCroppedPaper().setImageBitmap(croppedBitmap)
//                    iCropView.getPaper().visibility = View.GONE
//                    iCropView.getPaperRect().visibility = View.GONE
                    isCropped = true
                }
    }

//    fun showPreview() {
//
//        if (ActivityCompat.checkSelfPermission(context, Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
//            Toast.makeText(context, "please grant write file permission and trya gain", Toast.LENGTH_SHORT).show()
//        } else {
//            val dir = File(Environment.getExternalStorageDirectory(), IMAGES_DIR)
//            if (!dir.exists()) {
//                dir.mkdirs()
//            }
//
//            val cropPic = croppedBitmap
//            var mIntent = Intent()
//            if (null != cropPic) {
//                val file = File(dir, "crop_${SystemClock.currentThreadTimeMillis()}.jpeg")
//                val outStream = FileOutputStream(file)
//                cropPic.compress(Bitmap.CompressFormat.JPEG, 100, outStream)
//                outStream.flush()
//                outStream.close()
//                addImageToGallery(file.absolutePath, this.context)
////                Toast.makeText(context, "picture saved, path: ${file.absolutePath}", Toast.LENGTH_SHORT).show()
//                mIntent.putExtra("path",file.absolutePath)
//            }
//            (context as FlutterActivity).setResult(CROP_CODE,mIntent)
//            context.finish()
//        }
//    }

    fun rotateBitmap(angle : Float) {
        object : AsyncTask<String, String, String>() {
            var mbitmap: Bitmap? = null
            override fun doInBackground(vararg params: String?): String {
                val matrix = Matrix()
                matrix.postRotate(angle)
                mbitmap = Bitmap.createBitmap(croppedBitmap!!, 0, 0, croppedBitmap!!.width, croppedBitmap!!.height, matrix, true)
                if(isWidthMax){
                    if(mbitmap!!.height>mbitmap!!.width){
                        val matrix = Matrix()
                        matrix.postRotate(270f)
                        mbitmap = Bitmap.createBitmap(croppedBitmap!!, 0, 0, croppedBitmap!!.width, croppedBitmap!!.height, matrix, true)
                    }
                }
//                croppedBitmap = mbitmap
                return ""
            }

            override fun onPostExecute(result: String?) {
                super.onPostExecute(result)
//                iCropView.getCroppedPaper().scaleType = ImageView.ScaleType.FIT_XY
//                iCropView.getCroppedPaper().setImageBitmap(mbitmap)
                
                SourceManager.mBitmap = mbitmap
                // save bitmap after crop
//                showPreview()
                iCropView.goToNextActivity()
            }
        }.execute()
    }
}