package com.detectdoc.opencv_document_detection.preview

import android.Manifest
import android.content.ContentValues
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Bitmap
import android.os.Bundle
import android.os.Environment
import android.os.SystemClock
import android.provider.MediaStore
import android.view.WindowManager
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import com.detectdoc.opencv_document_detection.R
import com.detectdoc.opencv_document_detection.SourceManager
import com.detectdoc.opencv_document_detection.crop.IMAGES_DIR
import org.w3c.dom.Text
import java.io.File
import java.io.FileOutputStream
import java.lang.Exception

class PreviewActivity : AppCompatActivity() {
    lateinit var imageViewId: ImageView
    lateinit var textViewDone: TextView
    lateinit var relativePreviewBack: RelativeLayout
    lateinit var txtReview: TextView
    override fun onCreate(savedInstanceState: Bundle?) {
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_preview)
        imageViewId = findViewById(R.id.imageViewId)
        txtReview = findViewById(R.id.txtReview)
        textViewDone = findViewById(R.id.textViewDone)
        relativePreviewBack = findViewById(R.id.relativePreviewBack)
        var intent = intent
        if (intent != null) {
            txtReview.text = intent.getStringExtra("titleReviewHeaderText")
            textViewDone.text = intent.getStringExtra("rightConfirmButtonText")
        }
        relativePreviewBack.setOnClickListener {
            finish()
        }
        textViewDone.setOnClickListener {
            saveImage()
        }
        try {
            actionBar!!.hide()
        } catch (ex: Exception) {
            println(ex)
        }
        imageViewId.setImageBitmap(SourceManager.mBitmap)
    }
    fun saveImage() {
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
//            Toast.makeText(this, "please grant write file permission and trya gain", Toast.LENGTH_SHORT).show()
        } else {
            val dir = File(Environment.getExternalStorageDirectory(), IMAGES_DIR)
            if (!dir.exists()) {
                dir.mkdirs()
            }
            val cropPic = SourceManager.mBitmap
            var mIntent = Intent()
            if (null != cropPic) {
                val file = File(dir, "crop_${SystemClock.currentThreadTimeMillis()}.jpeg")
                val outStream = FileOutputStream(file)
                cropPic.compress(Bitmap.CompressFormat.JPEG, 100, outStream)
                outStream.flush()
                outStream.close()
                addImageToGallery(file.absolutePath, this)
                mIntent.putExtra("path", file.absolutePath)
            }
            setResult(2651, mIntent)
            finish()
        }
    }
    fun addImageToGallery(filePath: String, context: Context) {
        val values = ContentValues()
        values.put(MediaStore.Images.Media.DATE_TAKEN, System.currentTimeMillis())
        values.put(MediaStore.Images.Media.MIME_TYPE, "image/jpeg")
        values.put(MediaStore.MediaColumns.DATA, filePath)
        context.contentResolver.insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values)
    }
}
