package com.detectdoc.opencv_document_detection.scan

import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Bundle
import android.os.Handler
import android.util.DisplayMetrics
import android.util.Log
import android.view.View
import android.view.WindowManager
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import com.detectdoc.opencv_document_detection.Constants
import com.detectdoc.opencv_document_detection.R
import com.detectdoc.opencv_document_detection.SourceManager
import com.detectdoc.opencv_document_detection.crop.CropActivity
import com.detectdoc.opencv_document_detection.processor.Corners
import org.opencv.android.CameraBridgeViewBase
import org.opencv.android.OpenCVLoader
import org.opencv.core.*
import org.opencv.imgproc.Imgproc
import org.opencv.objdetect.CascadeClassifier
import java.io.File
import java.io.FileOutputStream

class ScanActivity : AppCompatActivity() {

    val CARD_SCAN_CODE = 2500
    val CROP_CODE = 1520

    companion object {
        var mScanActivity: ScanActivity? = null
    }

    private var openCvCameraView: CameraBridgeViewBase? = null
    private var cascadeClassifier: CascadeClassifier? = null
    private var grayscaleImage: Mat? = null
    private var absoluteFaceSize: Int = 0
    protected val TAG = this.javaClass.simpleName
    private var isFaceDetected = false
    private var isCardDetected = false

    private val REQUEST_CAMERA_PERMISSION = 0
    lateinit var image_scan_enabled: ImageView
    lateinit var image_scan_disabled: ImageView
    lateinit var image_close: ImageView
    lateinit var textViewForCam: TextView
    var mInputMainFrame: Mat? = null
    var isAnimating = false
    var tempSavedFrame: Mat? = null
    val displayMetrics = DisplayMetrics()
    lateinit var intent_message: Intent

    lateinit var titleEditScanHeaderText: String
    lateinit var  rightNextButtonText: String
    lateinit var titleReviewHeaderText: String
    lateinit var rightConfirmButtonText: String

    override fun onCreate(savedInstanceState: Bundle?) {
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_scan)

        mScanActivity = this

        image_scan_enabled = findViewById(R.id.image_scan_enabled)
        image_scan_disabled = findViewById(R.id.image_scan_disabled)
        image_close = findViewById(R.id.image_close)
        textViewForCam = findViewById(R.id.textViewForCam)


        intent_message = intent
        if (intent_message != null) {
            titleEditScanHeaderText = intent_message.getStringExtra("titleEditScanHeaderText")
            rightNextButtonText = intent_message.getStringExtra("rightNextButtonText")
            titleReviewHeaderText = intent_message.getStringExtra("titleReviewHeaderText")
            rightConfirmButtonText = intent_message.getStringExtra("rightConfirmButtonText")
            textViewForCam.text = intent_message.getStringExtra("message")
        }

        windowManager.defaultDisplay.getMetrics(displayMetrics)

        // performs good but slow, Purely depending RAM of device
//        if(displayMetrics.widthPixels<=1300) {
//            Constants.width = displayMetrics.widthPixels
//            Constants.height = displayMetrics.heightPixels
//        }
//        else{
//            Constants.width = 1080
//            Constants.height = 1920
//        }

        // Fast but output will be little
        Constants.width = 720
        Constants.height = 1280

        try {
            actionBar!!.hide()
        } catch (ex: Exception) {
            println(ex)
        }

        image_close.setOnClickListener {
            onBackPressed()
        }

        prepare()
    }

    override fun onBackPressed() {
        setResult(CARD_SCAN_CODE, null)
        finish()
    }

    fun prepare() {
        if (!OpenCVLoader.initDebug()) {
            Log.i(TAG, "loading opencv error, exit")
            finish()
        }
        if (ContextCompat.checkSelfPermission(this, android.Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED &&
                ContextCompat.checkSelfPermission(this, android.Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, arrayOf(android.Manifest.permission.CAMERA, android.Manifest.permission.WRITE_EXTERNAL_STORAGE), REQUEST_CAMERA_PERMISSION)
        } else if (ContextCompat.checkSelfPermission(this, android.Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, arrayOf(android.Manifest.permission.CAMERA), REQUEST_CAMERA_PERMISSION)
        } else if (ContextCompat.checkSelfPermission(this, android.Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, arrayOf(android.Manifest.permission.WRITE_EXTERNAL_STORAGE), REQUEST_CAMERA_PERMISSION)
        } else {
            init()
        }
    }


    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        if (requestCode == REQUEST_CAMERA_PERMISSION
                && (grantResults[permissions.indexOf(android.Manifest.permission.CAMERA)] == PackageManager.PERMISSION_GRANTED)) {
            init()
        }
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
    }

    fun init() {
        openCvCameraView = findViewById(R.id.javaCameraView)
        openCvCameraView!!.setMaxFrameSize(Constants.height, Constants.width)
        openCvCameraView!!.setCvCameraViewListener(object : CameraBridgeViewBase.CvCameraViewListener {
            override fun onCameraViewStarted(width: Int, height: Int) {
                grayscaleImage = Mat(height, width, CvType.CV_8UC3)

                // The faces will be a 20% of the height of the screen
                absoluteFaceSize = (height * 0.15).toInt()
            }

            override fun onCameraViewStopped() {
            }

            override fun onCameraFrame(aInputFrame: Mat?): Mat {
                mInputMainFrame = aInputFrame
                if (aInputFrame == null) {
                    return Mat()
                }

                Imgproc.cvtColor(aInputFrame, grayscaleImage, Imgproc.COLOR_RGBA2RGB)

                val faces = MatOfRect()
                // Use the classifier to detect faces
                if (cascadeClassifier != null) {
                    cascadeClassifier!!.detectMultiScale(grayscaleImage, faces, 5.5, 3, 2,
                            Size(absoluteFaceSize.toDouble(), absoluteFaceSize.toDouble()), Size())
/*
                    cascadeClassifier!!.detectMultiScale(grayscaleImage, faces, 1.1, 2, 2,
                            Size(absoluteFaceSize.toDouble(), absoluteFaceSize.toDouble()), Size())
*/
                }

                // If there are any faces found, draw a rectangle around it
                val facesArray = faces.toArray()
                if (facesArray.size > 0) {
                    isFaceDetected = true
                    findROI(aInputFrame)

                } else {
                    isFaceDetected = false

                }

                runOnUiThread {
                    var needToshow = false
                    Log.e("*********", isCardDetected.toString() + " AND " + isFaceDetected.toString())
//                    println(isCardDetected + " "+ isFaceDetected)
                    if (isCardDetected && isFaceDetected) {
                        needToshow = true
                    } else {
                        needToshow = false
                    }

                    if (!isAnimating) {
                        isAnimating = true

                        if (needToshow) {
                            image_scan_enabled.animate().alpha(1f).setDuration(400).start()
//                            image_scan_enabled.visibility = View.VISIBLE
                        } else {
                            image_scan_enabled.animate().alpha(0f).setDuration(400).start()
//                            image_scan_enabled.visibility = View.INVISIBLE
                        }

                        Handler().postDelayed(object : Runnable {
                            override fun run() {
                                isAnimating = false

                            }
                        }, 400)
                    }
                }
                tempSavedFrame = aInputFrame
                return aInputFrame
            }

        })
        initializeOpenCVDependencies()

        image_scan_enabled.setOnClickListener(object : View.OnClickListener {
            override fun onClick(v: View?) {
                if (isCardDetected && isFaceDetected) {
                    onPictureCapture()
                }
            }
        })
    }

    private fun initializeOpenCVDependencies() {
        try {
            // Copy the resource into a temp file so OpenCV can load it
            val inputStream = resources.openRawResource(R.raw.lbpcascade_frontalface)
            val cascadeDir = getDir("cascade", Context.MODE_PRIVATE)
            val mCascadeFile = File(cascadeDir, "lbpcascade_frontalface.xml")
            val os = FileOutputStream(mCascadeFile)


            val buffer = ByteArray(4096)
            var bytesRead = inputStream.read(buffer)

            while (bytesRead != -1) {
                os.write(buffer, 0, bytesRead)
                bytesRead = inputStream.read(buffer)
            }
            inputStream.close()
            os.close()

            // Load the cascade classifier
            cascadeClassifier = CascadeClassifier(mCascadeFile.getAbsolutePath())
        } catch (e: Exception) {
            Log.e("OpenCVActivity", "Error loading cascade", e)
        }

        // And we are ready to go
        openCvCameraView!!.enableView()
    }

    fun showMessage(id: Int) {
        Toast.makeText(this, id, Toast.LENGTH_SHORT).show();
    }

    private fun findROI(sourceMat: Mat) {//}: Mat {
        val mat = Mat()
        sourceMat.copyTo(mat)

        Imgproc.cvtColor(mat, mat, Imgproc.COLOR_RGB2GRAY)
        Imgproc.threshold(mat, mat, 146.0, 250.0, Imgproc.THRESH_BINARY)

        // find contours
        val contours = ArrayList<MatOfPoint>()
        val boundingRects = ArrayList<RotatedRect>()
        Imgproc.findContours(mat, contours, Mat(), Imgproc.RETR_LIST, Imgproc.CHAIN_APPROX_SIMPLE)

        // find appropriate bounding rectangles
        for (contour in contours) {
            val areaPoints = MatOfPoint2f(*contour.toArray())
            val boundingRect = Imgproc.minAreaRect(areaPoints)
            boundingRects.add(boundingRect)
        }

        val documentRect = getBestRectByArea(boundingRects)
        if (documentRect != null) {
            isCardDetected = true
            val rect_points = arrayOfNulls<Point>(4)
            val rectPointsToSet = ArrayList<Point?>()
            documentRect!!.points(rect_points)
            for (i in 0..3) {
                Imgproc.line(sourceMat, rect_points[i], rect_points[(i + 1) % 4], Scalar(255.0, 255.0, 255.0, 1.0), 2)
                Log.e("****" + i, rect_points[i].toString() + " " + rect_points[(i + 1) % 4].toString())
                rectPointsToSet.add(rect_points[i])
            }
            SourceManager.corners = Corners(rectPointsToSet, Size(Constants.height.toDouble(), Constants.width.toDouble()))
        } else {
            isCardDetected = false
        }
    }

    fun getBestRectByArea(boundingRects: List<RotatedRect>): RotatedRect? {
        var bestRect: RotatedRect? = null

        if (boundingRects.size >= 1) {
            var boundingRect: RotatedRect
            var vertices = arrayOfNulls<Point>(4)

            var rect: Rect
            var maxArea: Double = 0.0;
            var ixMaxArea = 0

            // find best rect by area
            boundingRect = boundingRects[ixMaxArea]
            boundingRect.points(vertices)

            rect = Imgproc.boundingRect(MatOfPoint(*vertices))
            maxArea = rect.area()

            for (ix in 1 until boundingRects.size) {
                boundingRect = boundingRects[ix]
                boundingRect.points(vertices)
                rect = Imgproc.boundingRect(MatOfPoint(*vertices))

                if (rect.area() > maxArea) {
                    maxArea = rect.area()
                    ixMaxArea = ix
                }
            }
            bestRect = boundingRects[ixMaxArea]
        }

        return bestRect
    }

    fun onPictureCapture() {
        if (mInputMainFrame != null) {
            SourceManager.pic = grayscaleImage
            var intent = Intent(this, CropActivity::class.java)
            intent.putExtra("titleEditScanHeaderText",titleEditScanHeaderText)
            intent.putExtra("rightNextButtonText",rightNextButtonText)
            intent.putExtra("titleReviewHeaderText",titleReviewHeaderText)
            intent.putExtra("rightConfirmButtonText",rightConfirmButtonText)
            startActivityForResult(intent,CROP_CODE)

//            startActivityForResult(Intent(this, CropActivity::class.java), CROP_CODE)
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == CROP_CODE) {
            if (data != null) {
                var mIntent = Intent()
                mIntent.putExtra("path", data!!.getStringExtra("path"))
                setResult(CARD_SCAN_CODE, mIntent)
                finish()
            }
        }
    }
}